#include "Dragon.h"

Dragon::Dragon()
{
}

Dragon::~Dragon()
{
  if(glIsList(draw_list))
    glDeleteLists(draw_list,1);

}

int Dragon::load_mesh(const char* filename)
{
  int ok=0;

  mesh.load_mesh_file(filename);
  mesh.fill_color(0.3,0.3,0.3);
  mesh.normalize_size();
  mesh.build_xz_texture_coordinate(2,2);
  mesh.scale(0.03*30);
  mesh.build_normal();
  mesh.translate(0.00,WATER_LEVEL+0.1,0.0);

  return ok;
}


int Dragon::draw(Position *position)
{

  float R2=12*12;
  float x_0 = 0;
  float z_0 = 0;

  //if close enough, draw it
  if( (position->get_x()-x_0)*(position->get_x()-x_0)+(position->get_z()-z_0)*(position->get_z()-z_0)<R2 )
      glCallList(draw_list);

  return 0;
}


int Dragon::set_texture(const char* texture_path)
{
  int ok=0;

  //**********************//
  //texture 
  //**********************//

  glEnable (GL_TEXTURE_2D);
  
  Texture tex;
  ok += tex.load_picture(texture_path);


  glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
  glGenTextures (1, &texture_number);
  glBindTexture (GL_TEXTURE_2D, texture_number);

  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

  gluBuild2DMipmaps (GL_TEXTURE_2D, 4, tex.get_Nx(), tex.get_Ny(), GL_RGBA,
                         GL_UNSIGNED_BYTE, tex.get_data());
  glTexEnvi (GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_REPLACE);


  ok += build_draw_list();
  
  return ok;
}

int Dragon::build_draw_list()
{
  int ok=0;

  if(glIsList(draw_list))
    glDeleteLists(draw_list,1);
  draw_list = glGenLists(1);
  glNewList(draw_list,GL_COMPILE);





  glColor4f(0.6,0.8,0.7,1.0);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,texture_number);
  
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  
  glVertexPointer(3,GL_FLOAT,0,mesh.get_vertex());
  glNormalPointer(GL_FLOAT,0,mesh.get_normal());
  glTexCoordPointer(2,GL_FLOAT,0,mesh.get_texture_coordinate());
  
  glDrawElements(GL_TRIANGLES,mesh.get_N_triangle()*3,GL_UNSIGNED_INT,mesh.get_connectivity());
  
  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  
  glColor4f(1.0,1.0,1.0,1.0);
  glDisable(GL_TEXTURE_2D);

  glEndList();


  return ok;
}
