
#include "Grass.h"

Grass::Grass()
{
  position  = NULL;
  scale     = NULL;
  cos_theta = NULL;
  sin_theta = NULL;
  N=0;
}

Grass::~Grass()
{
  destroy();
}

int Grass::destroy()
{
  if(position!=NULL)
    {
      delete []position;
      position=NULL;
    }
  if(scale!=NULL)
    {
      delete []scale;
      scale=NULL;
    }
  if(cos_theta!=NULL)
    {
      delete []cos_theta;
      cos_theta=NULL;
    }
  if(sin_theta!=NULL)
    {
      delete []sin_theta;
      sin_theta=NULL;
    }


  return 0;
}

int Grass::set_number(int _N)
{
  int ok=0;

  N=_N;
  if(N>0)
    {
      position = new float[3*N];
      scale = new float[2*N];
      cos_theta = new float[N];
      sin_theta = new float[N];

      if(position==NULL || scale==NULL || cos_theta==NULL || sin_theta==NULL)
	{
	  cout<<"ERROR allocation position in Grass\n";
	  exit(-1);
	}


      return ok;
    }
  else
    return(-1);
}

int Grass::set_position(Ground *ground)
{
  int ok=0;
  int k=0;

  //get dimensions
  //************************//
  float max_x = ground->get_size_x();
  float max_y = ground->get_size_y();

  float theta=0.0;

  for(k=0;k<N;k++)
    {
      position[3*k+0] = (float)(rand()%PRECISION_RAND)/(PRECISION_RAND)*max_x;
      position[3*k+2] = (float)(rand()%PRECISION_RAND)/(PRECISION_RAND)*max_y;
      position[3*k+1] = ground->get_height(position[3*k+0],position[3*k+2]);

      scale[2*k+0] = ((float)(rand()%PRECISION_RAND)/(PRECISION_RAND)+1)*0.5;
      scale[2*k+1] = ((float)(rand()%PRECISION_RAND)/(PRECISION_RAND)+1)*0.5;

      //random orientation
      //save the orientation in a LUT to avoid calculating a cos and sin during the drawing loop
      theta = (float)(k%180)/180.0*PI;
      cos_theta[k]=cos(theta);
      sin_theta[k]=sin(theta);

      //no grass on the water
      if(position[3*k+1] <= WATER_LEVEL+0.1 || position[3*k+1]>2.2)
	 k--;

    }


  return ok;
}


int Grass::set_texture(const char* texture_path)
{
  int ok=0;

  //**********************//
  //texture 
  //**********************//

  glEnable (GL_TEXTURE_2D);
  
  Texture tex;
  ok += tex.load_picture(texture_path);


  glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
  glGenTextures (1, &texture_number);
  glBindTexture (GL_TEXTURE_2D, texture_number);

  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

  gluBuild2DMipmaps (GL_TEXTURE_2D, 4, tex.get_Nx(), tex.get_Ny(), GL_RGBA,
                         GL_UNSIGNED_BYTE, tex.get_data());
  glTexEnvi (GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  
  return ok;
}



int Grass::draw(GLfloat *matrix,Position *X,int kt)
{
  int ok=0;
  int k=0;
  float R=3*3;
  float current_z=0;

  float deform_x=0.0,deform_z=0.0;
  float current_cos=1.0,current_sin=0.0;

  glDisable(GL_LIGHTING);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,texture_number);


  glColor4f(0.65,0.65,0.65,1.0);
  for(k=0;k<N;k++)
    {
      current_z = matrix[2]*position[3*k]+matrix[6]*position[3*k+1]+matrix[10]*position[3*k+2]+matrix[14];
      if( current_z<0 && (position[3*k]-X->get_x())*(position[3*k]-X->get_x())+(position[3*k+2]-X->get_z())*(position[3*k+2]-X->get_z())<R)
	{

	  //set deformation
	  deform_x=cos((float)kt/10-position[3*k]*2*0.6-position[3*k+2]*1.24*0.6)*0.05/5;
	  deform_z = cos((float)kt/9.6-position[3*k]*2.4*0.6)*0.03/5;


	  //set orientation
 	  current_cos = cos_theta[k];
 	  current_sin = sin_theta[k];


	  glPushMatrix();

	  //translate and scale
	  glTranslatef(position[3*k],position[3*k+1],position[3*k+2]);
	  glScalef(scale[0],scale[1],scale[0]);


	  //draw triangles
	  glBegin(GL_TRIANGLE_STRIP);
	  glTexCoord2f(0.0,1.0); glVertex3f(
					    -0.003*current_cos,
					    0.00,
					    -0.003*current_sin
					    );
	  glTexCoord2f(1.0,1.0); glVertex3f(
					    +0.003*current_cos,
					    0.00,
					    +0.003*current_sin
					    );
	  glTexCoord2f(0.2,0.3); glVertex3f(
					    -0.0015*current_cos+deform_x*0.5,
					    0.015,
					    -0.0015*current_sin+deform_z*0.5
					    );
	  glTexCoord2f(0.8,0.3); glVertex3f(
					    +0.0015*current_cos+deform_x*0.5,
					    0.015,
					    +0.0015*current_sin+deform_z*0.5
					    );
	  glTexCoord2f(0.5,0.0); glVertex3f( 
					    0.004*current_sin+deform_x,
					    0.02,
					    0.004*current_cos+deform_z
					    );
	  glEnd();

	  glPopMatrix();
	}
    }
  glColor4f(1.0,1.0,1.0,1.0);
  glDisable(GL_TEXTURE_2D);
  glEnable(GL_LIGHTING);

  return ok;
}


