
#include "Ground.h"



Ground::Ground()
{
  N_x=0;N_y=0;
  for(int k=0;k<3;scale[k]=1,k++);
  height_map=NULL;
  list_number=0;
}

Ground::~Ground()
{
  destroy();
}

int Ground::destroy()
{
  N_x=0;N_y=0;
  for(int k=0;k<3;scale[k]=1,k++);

  if(height_map!=NULL)
    delete[] height_map;
  if(glIsList(list_number))
    glDeleteLists(list_number,1);

  return 0;
}

int Ground::set_scale(float scale_x,float scale_y,float scale_z)
{
  int ok=0;

  scale[0]=scale_x;
  scale[1]=scale_y;
  scale[2]=scale_z;

  return ok;
}

int Ground::build(int _N_x,int _N_y,const char* height_field_path)
{
  int ok=0;

  N_x=_N_x;
  N_y=_N_y;


  //**************************//
  //INITIALIZE MESH
  //**************************//
  ok += mesh.set_parameterized_size(N_x,N_y);
  ok += mesh.build_parameterized_connectivity(N_x,N_y);
  //  ok += mesh.map_linear_texture_coordinate(N_x,N_y,1,1);
  ok += mesh.map_linear_multitexture_coordinate(N_x,N_y,1,1,50,50);

  //**************************//
  //load height_field
  //**************************//
  Texture height_field;
  ok += height_field.load_picture(height_field_path);
  


  float interpolated_texel[3];
  float current_vertex[3];
  int k_x=0,k_y=0;
  
  height_map = new float[N_x*N_y];
  if(height_map==NULL)
    {
      cout<<"ERROR height map not allocated\n";
      exit(-1);
    }


  for(k_x=0;k_x<N_x;k_x++)
    for(k_y=0;k_y<N_y;k_y++)
      {

	height_field.get_linear_texel((float)k_x/N_x,(float)k_y/N_y,interpolated_texel);
	//take only red component in account for height field
	current_vertex[0] = (float)k_x/(N_x-1)*scale[0];
	current_vertex[1] = (float)interpolated_texel[0]/256*scale[1];
	current_vertex[2] = (float)k_y/(N_y-1)*scale[2];


	//fill the mesh with the vertex
	mesh.fill_vertex(k_x,k_y,N_x,current_vertex);
	//fill also the height map
	height_map[k_x+k_y*N_x] = current_vertex[1];
      }


  //**************************//
  //NORMAL and normalize mesh
  //**************************//
  ok += mesh.build_normal();
  ok += mesh.fill_color(1.0,1.0,1.0);


  //**************************//
  //Create Call List
  //**************************//
  ok += create_call_list();


  return ok;
}

int Ground::draw()
{
  glCallList(list_number);

  return 0;
}

int Ground::create_call_list()
{
  int ok=0;

  //create the drawing list
  if(glIsList(list_number))
    glDeleteLists(list_number,1);
  list_number = glGenLists(1);
  glNewList(list_number,GL_COMPILE);




  //set memory for vertex,normal and texture


  //[texture_1 texture_2]
  float *multitexturing=mesh.get_texture_coordinate();


  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);



  //enable multitexturing
  glActiveTextureARB(GL_TEXTURE0_ARB);
  glBindTexture(GL_TEXTURE_2D,texture_number[0]);
  glEnable(GL_TEXTURE_2D);
  glClientActiveTextureARB(GL_TEXTURE0_ARB);
  glTexCoordPointer(2,GL_FLOAT,0,&multitexturing[0]);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);

  glActiveTextureARB(GL_TEXTURE1_ARB);
  glBindTexture(GL_TEXTURE_2D,texture_number[1]);
  glEnable(GL_TEXTURE_2D);
  glClientActiveTextureARB(GL_TEXTURE1_ARB);
  glTexCoordPointer(2,GL_FLOAT,0,&multitexturing[2*N_x*N_y]);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);





  //set vertex and normal
  glVertexPointer(3,GL_FLOAT,0,mesh.get_vertex());
  glNormalPointer(GL_FLOAT,0,mesh.get_normal());

  //draw
  glDrawElements(GL_TRIANGLES,mesh.get_N_triangle()*3,GL_UNSIGNED_INT,mesh.get_connectivity());

  //disable
  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);

  glActiveTextureARB(GL_TEXTURE1_ARB);
  glClientActiveTextureARB(GL_TEXTURE1_ARB);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisable(GL_TEXTURE_2D);

  glActiveTextureARB(GL_TEXTURE0_ARB);
  glClientActiveTextureARB(GL_TEXTURE0_ARB);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  glDisable(GL_TEXTURE_2D);


  glEndList();




















  //SAVE
//   //create the drawing list
//   if(glIsList(list_number))
//     glDeleteLists(list_number,1);
//   list_number = glGenLists(1);
//   glNewList(list_number,GL_COMPILE);


//   glEnable(GL_TEXTURE_2D);
//   glBindTexture(GL_TEXTURE_2D,texture_number[0]);


// //   glEnable(GL_BLEND);
// //   glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);



//   //set memory for vertex,normal and texture
//   glEnableClientState(GL_VERTEX_ARRAY);
//   glEnableClientState(GL_NORMAL_ARRAY);
//   glEnableClientState(GL_TEXTURE_COORD_ARRAY);

//   glVertexPointer(3,GL_FLOAT,0,mesh.get_vertex());
//   glTexCoordPointer(2,GL_FLOAT,0,mesh.get_texture_coordinate());
//   glNormalPointer(GL_FLOAT,0,mesh.get_normal());

//   //draw
//   glDrawElements(GL_TRIANGLES,mesh.get_N_triangle()*3,GL_UNSIGNED_INT,mesh.get_connectivity());

//   glDisableClientState(GL_VERTEX_ARRAY);
//   glDisableClientState(GL_NORMAL_ARRAY);
//   glDisableClientState(GL_TEXTURE_COORD_ARRAY);

//   //  glDisable(GL_BLEND);

//   glEndList();








  return ok;
}

int Ground::set_height_field_texture(const char* texture_path_1,const char* texture_path_2)
{
  int ok=0;

  //**********************//
  //texture 
  //**********************//

  glEnable (GL_TEXTURE_2D);
  
  Texture tex;
  ok += tex.load_picture(texture_path_1);


  glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
  glGenTextures (1, &(texture_number[0]));
  glBindTexture (GL_TEXTURE_2D, (texture_number[0]));

  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

  gluBuild2DMipmaps (GL_TEXTURE_2D, 4, tex.get_Nx(), tex.get_Ny(), GL_RGBA,
                         GL_UNSIGNED_BYTE, tex.get_data());
  glTexEnvi (GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_REPLACE);


  //grass2
  glEnable (GL_TEXTURE_2D);
  ok += tex.load_picture(texture_path_2);
  glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
  glGenTextures (1, &(texture_number[1]));
  glBindTexture (GL_TEXTURE_2D, (texture_number[1]));
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  gluBuild2DMipmaps (GL_TEXTURE_2D, 4, tex.get_Nx(), tex.get_Ny(), GL_RGBA,
                         GL_UNSIGNED_BYTE, tex.get_data());
  glTexEnvi (GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  
  return ok;
}


//linear interpolation of the height
float Ground::get_height(float _x,float _y)
{
  float u_0=0.0,u_1=0.0;
  int k_0=0,k_1=0;


  u_0 = _x*(N_x-1)/scale[0];
  u_1 = _y*(N_y-1)/scale[2];
  
  k_0 = (int)u_0;
  k_1 = (int)u_1;
  
  float t_x=(u_0-k_0);
  float t_y=(u_1-k_1);



  int X00 = (k_0+0) + (k_1+0)*N_x;
  int X01 = (k_0+1) + (k_1+0)*N_x;
  int X10 = (k_0+0) + (k_1+1)*N_x;
  int X11 = (k_0+1) + (k_1+1)*N_x;




  float h=0.0;
  h =   t_x *     t_y *height_map[X11]+
    (1-t_x) *     t_y *height_map[X10]+
       t_x  * (1-t_y) *height_map[X01]+
    (1-t_x) * (1-t_y) *height_map[X00];

  if(h>WATER_LEVEL)
    return h;
  else
    return WATER_LEVEL;
}



//linear interpolation of the normal
int Ground::get_normal(float _x,float _y,float *n)
{
  int ok=0;

  float u_0=0.0,u_1=0.0;
  int k_0=0,k_1=0;

  float *normal=mesh.get_normal();
  if(normal==NULL)
    exit(-1);

  u_0 = _x*(N_x-1)/scale[0];
  u_1 = _y*(N_y-1)/scale[2];
  
  k_0 = (int)u_0;
  k_1 = (int)u_1;
  
  float t_x=(u_0-k_0);
  float t_y=(u_1-k_1);

  int X00 = (k_0+0) + (k_1+0)*N_x;
  int X01 = (k_0+1) + (k_1+0)*N_x;
  int X10 = (k_0+0) + (k_1+1)*N_x;
  int X11 = (k_0+1) + (k_1+1)*N_x;


  int k_dim=0;
  for(k_dim=0;k_dim<3;k_dim++)
    {
      n[k_dim] =   t_x *     t_y *normal[3*X11+k_dim]+
	(1-t_x) *     t_y *normal[3*X10+k_dim]+
	t_x  * (1-t_y) *normal[3*X01+k_dim]+
	(1-t_x) * (1-t_y) *normal[3*X00+k_dim];
    }
  float norm=0;
  for(k_dim=0;k_dim<3;norm+=n[k_dim]*n[k_dim],k_dim++);
  for(k_dim=0;k_dim<3;n[k_dim]/=norm,k_dim++);
  

  return ok;
  
}



float Ground::get_size_x()
{
  return scale[0];
}

float Ground::get_size_y()
{
  return scale[2];
}
