#include "Map.h"

Map::Map()
{
}

Map::~Map()
{
  if(glIsList(draw_list))
    glDeleteLists(draw_list,1);

}


int Map::set_texture(const char* texture_path)
{
  int ok=0;

  //**********************//
  //texture 
  //**********************//

  glEnable (GL_TEXTURE_2D);
  
  Texture tex;
  ok += tex.load_picture(texture_path);


  glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
  glGenTextures (1, &texture_number);
  glBindTexture (GL_TEXTURE_2D, texture_number);

  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

  gluBuild2DMipmaps (GL_TEXTURE_2D, 4, tex.get_Nx(), tex.get_Ny(), GL_RGBA,
                         GL_UNSIGNED_BYTE, tex.get_data());
  glTexEnvi (GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  

  ok += build_draw_list();

  return ok;

}

int Map::draw(Position *position)
{
  int ok=0;

  //set matrix to identity (no projection)
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  glOrtho(-1, 1,-1, 1,0.1,2);
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity ();
  glViewport (0, 3*position->get_height()/4, position->get_width()/4, position->get_height()/4);

  //diable Z buffer
  glDisable(GL_DEPTH_TEST);

  //draw Map
  glCallList(draw_list);


  //draw Point
  glPointSize(5.0);
  glBegin(GL_POINTS);
  //no problem of Z buffer because GL_DEPTH_TEST is disabled
  glVertex3f(-1+position->get_x()/40*2,1-position->get_z()/40*2, -1);
  glEnd();

  glEnable(GL_DEPTH_TEST);

  return ok;
}

int Map::build_draw_list()
{
  int ok=0;



  if(glIsList(draw_list))
    glDeleteLists(draw_list,1);
  draw_list = glGenLists(1);
  glNewList(draw_list,GL_COMPILE);



  glColor4f(1,1,1,0.75);
  glEnable(GL_TEXTURE_2D);
  glEnable(GL_BLEND);
  glBindTexture(GL_TEXTURE_2D,texture_number);
  glBegin(GL_QUADS);
  glTexCoord2f(0.0,1.0); glVertex3f(-1,-1,-1);
  glTexCoord2f(0.0,0.0); glVertex3f(-1, 1,-1);
  glTexCoord2f(1.0,0.0); glVertex3f( 1, 1,-1);
  glTexCoord2f(1.0,1.0); glVertex3f( 1,-1,-1);
  glEnd();
  glDisable(GL_BLEND);
  glDisable(GL_TEXTURE_2D);



  glEndList();



  return ok;
}
