
#include "Mesh_tool.h"


Mesh_tool::Mesh_tool()
{
  N_triangle=0;
  N_vertex=0;

  vertex=NULL;
  normal=NULL;
  color=NULL;
  connectivity=NULL;
  texture_coordinates=NULL;
}

Mesh_tool::~Mesh_tool()
{
  destroy();
}

int Mesh_tool::destroy()
{

  N_triangle=0;
  N_vertex=0;

  if(vertex!=NULL)
    {
      delete []vertex;
      vertex=NULL;
    }
  if(normal!=NULL)
    {
      delete []normal;
      normal=NULL;
    }
  if(color!=NULL)
    {
      delete []color;
      color=NULL;
    }
  if(connectivity!=NULL)
    {
      delete []connectivity;
      connectivity=NULL;
    }
  if(texture_coordinates!=NULL)
    {
      delete []texture_coordinates;
      texture_coordinates=NULL;
    }

  return 0;
}


int Mesh_tool::load_mesh_file(const char* filename)
{

  //the name of the file should not have . that does not delimitate the extension

  int ok=0;
  bool is_known_extension=false;

  //look for the extension
  char *p_filename=NULL;
  char extension[MAX_SIZE_EXTENSION];
  for(p_filename=(char*)filename; *p_filename!='.'; p_filename++);

  //fill the extension
  int k;
  for(k=0,p_filename++;*p_filename!='\0';p_filename++,k++)
    extension[k]=*p_filename;
  extension[k]='\0';

  //check for known extensions
  if(strcmp(extension,"ply")==0)
    {
      load_ply_file(filename);
      is_known_extension=true;
    }

  if(is_known_extension==false)
    {
      cout<<"[WARNING UNKNOWN EXTENSION FOR "<<filename<<"]\n";
      return -1;
    }
  return ok;
}


int Mesh_tool::load_ply_file(const char* filename)
{


  int ok=0;
  FILE *fid=NULL;




  //open the file
  fid=fopen(filename,"r");
  if(fid==NULL)
    {
      cout<<"ERROR THE FILE "<<filename<<" CANNOT BE OPENED\n";
      return -1;
    }


  //read header
  ok += read_ply_header(fid,filename);
  //allocation of mesh
  ok += allocate_mesh();
  // reading of data
  ok += read_ply_data(fid,filename);



  ok += fclose(fid);

  return ok;
}


int Mesh_tool::read_ply_header(FILE *fid,const char* filename)
{
  int ok=0;

  //temporary buffer
  char buffer[SIZE_BUFFER];

  //line corresponding to the number of vertex and face
  char vertex_line[]="element vertex";
  char face_line[]  ="element face";
  char end_header_line[]="end_header";


  //**********************************************//
  //SPECIFICATION READING
  //**********************************************//


  //read the specifications
  fscanf(fid,"ply\n");
  if(fscanf(fid,"format ascii %s\n",buffer)!=1)
    {
      cout<<"ERROR ply not in ascii format: not supported yet\n";
      return -1;
    }


  //**********************************************//
  //NUMBER OF VERTEX
  //**********************************************//

  //look for the line corresponding to the number of vertex
  do
    {
      fgets(buffer,SIZE_BUFFER,fid);
    }while(strncmp(buffer,vertex_line,strlen(vertex_line))!=0);//compare the begining of the string

  //read the number of vertex
  if(sscanf(buffer,"element vertex %d\n",&N_vertex)!=1)
    {
      cout<<"ERROR cannot read the number of vertex in the file"<<filename<<"\n";
      return -1;
    }

  //**********************************************//
  //NUMBER OF TRIANGLE
  //**********************************************//


  //look for the line corresponding to the number of triangles
  do
    {
      fgets(buffer,SIZE_BUFFER,fid);
    }while(strncmp(buffer,face_line,strlen(face_line))!=0);
  //read the number of face
  if(sscanf(buffer,"element face %d\n",&N_triangle)!=1)
    {
      cout<<"ERROR cannot read the number of triangle in the file"<<filename<<"\n";
      return -1;
    }

  //**********************************************//
  //END OF HEADER
  //**********************************************//


  //look for the end of header
  do
    {
      fgets(buffer,SIZE_BUFFER,fid);
    }while(strncmp(buffer,end_header_line,strlen(end_header_line))!=0);



  return ok;
}


int Mesh_tool::read_ply_data(FILE *fid,const char* filename)
{
  int ok=0;


  char buffer[SIZE_BUFFER];//temporary buffer
  int k=0;//counter

  float *p_vertex=NULL;//pointer on vertex 


  //***********************************************//
  //READ VERTEX
  //***********************************************//

  char buff_x[SIZE_BUFFER];//temporary buffer for x
  char buff_y[SIZE_BUFFER];//temporary buffer for y
  char buff_z[SIZE_BUFFER];//temporary buffer for z

  //for every vertex
  for(k=0,p_vertex=vertex;k<N_vertex;k++)
    {
      //read line
      fgets(buffer,SIZE_BUFFER,fid);

      //get (x,y,z)
      if(sscanf(buffer,"%s %s %s",(char*)&buff_x,(char*)&buff_y,(char*)&buff_z)!=3)
	{
	  cout<<"ERROR (x,y,z) not caught in vertex reading at line "<<k<<"\n";
	  return -1;
	}

      //convert char* to float and store in vertex table
      *(p_vertex++) = strtod((const char*)buff_x,NULL);
      *(p_vertex++) = strtod((const char*)buff_y,NULL);
      *(p_vertex++) = strtod((const char*)buff_z,NULL);

    }



  //***********************************************//
  //READ CONNECTIVITY
  //***********************************************//
  
  int *p_conn_1=connectivity + 0;
  int *p_conn_2=connectivity + 1;
  int *p_conn_3=connectivity + 2;

  for(k=0;k<N_triangle;k++,p_conn_1+=3,p_conn_2+=3,p_conn_3+=3)
    if(fscanf(fid,"3 %d %d %d\n",p_conn_1,p_conn_2,p_conn_3)!=3)
      {
	cout<<"ERROR cannot read connectivity at line "<<k<<"in file "<<filename<<"\n";
	return (-1);
      }


  return ok;
}


int Mesh_tool::allocate_mesh()
{
  int ok=0;

  //in case of a previous mesh
  int temp_n_vertex=N_vertex;
  int temp_n_triangle=N_triangle;

  ok += destroy();

  N_vertex = temp_n_vertex;
  N_triangle = temp_n_triangle;


  //ALLOCATION
  if(N_vertex>0 && N_triangle>0)
    {
      vertex = new float[3*N_vertex];
      normal = new float[3*N_vertex];
      color = new float[3*N_vertex];
      
      connectivity = new int[3*N_triangle];//3 vertex per triangle
    }


  //check if ok
  if(vertex==NULL || normal==NULL || color==NULL || connectivity==NULL)
    {
      cout<<"ERROR table not allocated, memory space or number of vertex/triangle problem\n";
      return (-1);
    }

  for(int k_fill=0;k_fill<N_vertex;vertex[k_fill]=-100,k_fill++);
    
  return ok;
}



//build normal at each vertex by averaging the normal of each faces
int Mesh_tool::build_normal()
{

  int ok=0;

  float temp_normal[3];
  float p1[3],p2[3],p3[3];//current positions
  float v1[3],v2[3];//current vectors
  float norm_1=0,norm_2=0;




  if(N_triangle<=0 || N_vertex<=0)
    return (-1);

  //init normal
  int k=0,k_dim=0;
  for(k=0;k<N_vertex;k++)
    for(k_dim=0;k_dim<3;k_dim++)
      normal[3*k+k_dim]=0;

  //loop in every triangle
  int k_triangle=0;
  for(k=0;k<N_triangle;k++)
    {
      //load current positions
      //(position = vertex[3*connectivity[3*k_triangle+k_vertex_number_in_triangle]+k_dimension])
      p1[0] = vertex[3*connectivity[3*k+0]]; p1[1] = vertex[3*connectivity[3*k+0]+1]; p1[2] = vertex[3*connectivity[3*k+0]+2];
      p2[0] = vertex[3*connectivity[3*k+1]]; p2[1] = vertex[3*connectivity[3*k+1]+1]; p2[2] = vertex[3*connectivity[3*k+1]+2];
      p3[0] = vertex[3*connectivity[3*k+2]]; p3[1] = vertex[3*connectivity[3*k+2]+1]; p3[2] = vertex[3*connectivity[3*k+2]+2];




      //create current vectors v1=v[21]=p2-p1 and v2=v[31]=p3-p1 and get the norm
      norm_1=0;norm_2=0;
      for(k_dim=0;k_dim<3;k_dim++)
	{
	  //vector
	  v1[k_dim] = p2[k_dim]-p1[k_dim];
	  v2[k_dim] = p3[k_dim]-p1[k_dim];

	  //norm
	  norm_1 += v1[k_dim]*v1[k_dim];
	  norm_2 += v2[k_dim]*v2[k_dim];
	}




      //normalize them
      for(norm_1=powf(norm_1,(float)0.5),norm_2=powf(norm_2,(float)0.5),k_dim=0;k_dim<3;k_dim++)
	{
	  v1[k_dim]/=norm_1;
	  v2[k_dim]/=norm_2;
	}


      //get normal to the face with the vectorial product Normal = v1/||v1|| x v2/||v2||
      temp_normal[0] = v1[1]*v2[2]-v1[2]*v2[1];
      temp_normal[1] = v1[2]*v2[0]-v1[0]*v2[2];
      temp_normal[2] = v1[0]*v2[1]-v1[1]*v2[0];
      

      //fill every vertex normal that are linked to this triangle
      //(the normal is already unitary so we can add them and normalize them later)
      for(k_triangle=0;k_triangle<3;k_triangle++)
	for(k_dim=0;k_dim<3;k_dim++)
	  normal[3*connectivity[3*k+k_triangle]+k_dim] += temp_normal[k_dim];
    }

  //Now, for every vertex, we normalize the normal (so we average them)
  float norm=0;
  for(k=0;k<N_vertex;k++)
    {
      //calculate norm
      for(k_dim=0,norm=0;k_dim<3;k_dim++)
	norm += normal[3*k+k_dim]*normal[3*k+k_dim];
      //divide by norm
      norm = powf(norm,(float)0.5);
      if(norm>0.0000001)
	{
	  for(k_dim=0;k_dim<3;k_dim++)
	    {
	      normal[3*k+k_dim]/=-norm;
	    }
	}
      else
	{
	  normal[3*k]=0;
	  normal[3*k+1]=0;
	  normal[3*k+2]=1;
	}


    }

  return ok;
  
}


//fill mesh with one color
int Mesh_tool::fill_color(float *_color)
{
  int ok=0;

  if(N_triangle<=0 ||N_vertex<=0)
    return(-1);

  //for every vertex, fill the color
  int k=0,k_dim=0;
  for(k=0;k<N_vertex;k++)
    for(k_dim=0;k_dim<3;k_dim++)
      color[3*k+k_dim]=_color[k_dim];
  return ok;
}
int Mesh_tool::fill_color(float r,float g,float b)
{
  float _color[3];
  _color[0]=r;
  _color[1]=g;
  _color[2]=b;
  return fill_color(_color);
}


int Mesh_tool::fill_color(int type)
{
  int ok=-1;
  if(type==COLOR_XYZ)
    {
      int k=0,k_dim=0;
      for(k=0;k<N_vertex;k++)
	for(k_dim=0;k_dim<3;k_dim++)
	  color[3*k+k_dim]=vertex[3*k+k_dim];
      
      ok=0;
    }

  return ok;
}


//normalize the size of the object to [-1,1]
int Mesh_tool::normalize_size()
{

  int ok=0;

  float mesh_min[3],mesh_max[3];//save of min and max

  if(N_vertex<=0 || N_triangle<=0)
    return(-1);


  //init the values of the min and max
  int k_dim=0;
  for(k_dim=0;k_dim<3;mesh_min[k_dim]=+INFINITY,mesh_max[k_dim]=-INFINITY,k_dim++);
  
  //loop on every vertex to know the min and max
  int k=0;
  float current_value=0;
  for(k=0;k<N_vertex;k++)
    {

      //load current position
      for(k_dim=0;k_dim<3;k_dim++)
	{
	  current_value = vertex[3*k+k_dim];

	  //update table of min and max
	  if(mesh_min[k_dim]>current_value)
	    mesh_min[k_dim]=current_value;
	  if(mesh_max[k_dim]<current_value)
	    mesh_max[k_dim]=current_value;
	}

    }

  //get the extremum size
  float L=0;
  L = fmax(fmax(mesh_max[0]-mesh_min[0],mesh_max[1]-mesh_min[1]),mesh_max[2]-mesh_min[2]);
  
  //normalize all the vertex
  for(k=0;k<N_vertex;k++)
    for(k_dim=0;k_dim<3;k_dim++)
      vertex[3*k+k_dim] = 2*((vertex[3*k+k_dim]-mesh_min[k_dim])/L-0.5*(mesh_max[k_dim]-mesh_min[k_dim])/L);

  //the normal need to be recalculated

  return ok;
  
}


//scale the Mesh by (scale_x,scale_y,scale_z)
int Mesh_tool::scale(float *scale)
{
  int ok=0;

  if(N_triangle<=0 || N_vertex<=0)
    return(-1);

  //scale every vertex
  int k=0,k_dim=0;
  for(k=0;k<N_vertex;k++)
    for(k_dim=0;k_dim<3;k_dim++)
      vertex[3*k+k_dim] *= scale[k_dim];


  return ok;
}

//uniform scale
int Mesh_tool::scale(float _scale)
{
  int ok=0;

  float v_scale[3];
  int k_dim=0;
  for(k_dim=0; k_dim<3; v_scale[k_dim]=_scale,k_dim++);
  
  scale(v_scale);

  return ok;
}

//translate the Mesh by (translate_x,translate_y,translate_z)
int Mesh_tool::translate(float *translate)
{
  int ok=0;

  if(N_triangle<=0 || N_vertex<=0)
    return(-1);

  //scale every vertex
  int k=0,k_dim=0;
  for(k=0;k<N_vertex;k++)
    for(k_dim=0;k_dim<3;k_dim++)
      vertex[3*k+k_dim] += translate[k_dim];


  return ok;
}

int Mesh_tool::translate(float t_x,float t_y,float t_z)
{
  float _translate[3];
  
  _translate[0] = t_x;
  _translate[1] = t_y;
  _translate[2] = t_z;

  return translate(_translate);
}

//rotate the vertex around the axe "axe" with the angle "angle"
//see the expression in wikipedia (comes from Rodrigez formula)
int Mesh_tool::rotate(float angle,float *axe)
{
  int ok=0;

  //matrix of rotation
  float rotation_matrix[9];


  if(N_vertex<=0)
    return(-1);

  //normalize the axe
  float norm=0;
  int k_dim=0;
  for(k_dim=0;k_dim<3;norm+=axe[k_dim]*axe[k_dim],k_dim++);
  for(k_dim=0,norm=powf(norm,(float)0.5);k_dim<3;axe[k_dim]/=norm,k_dim++);


  //fill the rotation matrix
  ok += fill_rotation_matrix(angle,axe,rotation_matrix);



  //rotate every vertex
  int k=0;
  float temp_vertex[3];
  for(k=0;k<N_vertex;k++)
    {
      for(k_dim=0;k_dim<3;k_dim++)
	temp_vertex[k_dim] = vertex[3*k+k_dim];
      ok += matrix_product(temp_vertex,rotation_matrix,&(vertex[3*k]));

    }
  

  
  return ok;
}


int Mesh_tool::rotate(float angle,float r_x,float r_y,float r_z)
{
  float axe[3];
  axe[0]= r_x;
  axe[1]= r_y;
  axe[2]= r_z;

  return rotate(angle,axe);
}

int Mesh_tool::fill_rotation_matrix(float angle,float *axe,float *rotation_matrix)
{
  int ok=0;
  //suppose the axe is normalized

  //get cos and sin of the angle
  float cos_alpha = cos(angle);
  float sin_alpha = sin(angle);

  //expression used many times
  float m_cos = 1-cos_alpha;

  rotation_matrix[0] = cos_alpha+m_cos*axe[0]*axe[0];
  rotation_matrix[1] = m_cos*axe[1]*axe[0]+sin_alpha*axe[2];
  rotation_matrix[2] = m_cos*axe[2]*axe[0]-sin_alpha*axe[1];
  rotation_matrix[3] = m_cos*axe[0]*axe[1]-sin_alpha*axe[2];
  rotation_matrix[4] = cos_alpha+m_cos*axe[1]*axe[1];
  rotation_matrix[5] = m_cos*axe[2]*axe[1]+sin_alpha*axe[0];
  rotation_matrix[6] = m_cos*axe[0]*axe[2]+sin_alpha*axe[1];
  rotation_matrix[7] = m_cos*axe[1]*axe[2]-sin_alpha*axe[0];
  rotation_matrix[8] = cos_alpha+m_cos*axe[2]*axe[2];


  
  return ok;
}


//do matrix product y=Mx
int Mesh_tool::matrix_product(float *x,float *M,float *y)
{
  int ok=0;

  int k1=0,k2=0;
  for(k1=0; k1<3; k1++)
    for(y[k1]=0,k2=0; k2<3; k2++)
      y[k1] += M[k1+3*k2]*x[k2];

  return ok;
}





float* Mesh_tool::get_vertex()
{
  return vertex;
}
float* Mesh_tool::get_normal()
{
  return normal;
}
float* Mesh_tool::get_color()
{
  return color;
}
float* Mesh_tool::get_texture_coordinate()
{
  return texture_coordinates;
}

int* Mesh_tool::get_connectivity()
{
  return connectivity;
}
int Mesh_tool::get_N_vertex()
{
  return N_vertex;
}
int Mesh_tool::get_N_triangle()
{
  return N_triangle;
}


int Mesh_tool::create_cylinder(int N_theta,int N_h,float radius,float H)
{
  int ok=0;
  int k_theta=0;
  int k_h;
  float theta=0.0;
  float h;
  int k=0;

  N_vertex = N_theta*N_h;
  N_triangle = 2*(N_theta-1)*(N_h-1);


  ok += allocate_mesh();
  if(ok!=0)
    exit(-1);



  for(k_theta=0;k_theta<N_theta;k_theta++)
    {
      //current theta
      theta = (float)k_theta*2*PI/(N_theta-1);
      for(k_h=0;k_h<N_h;k_h++)
	{
	  //current phi
	  h = (float)k_h/(N_h-1)*H;
	  //position of the vertex vector
	  k = 3*(k_theta+k_h*N_theta);

	  vertex[k]   = radius*cos(theta);
	  vertex[k+1] = h;
	  vertex[k+2] = radius*sin(theta);

	}
    }


  for(k=0,k_theta=0;k_theta<N_theta-1;k_theta++)
    {
      for(k_h=0;k_h<N_h-1;k_h++)
	{

	  //first way
	  connectivity[k] = (k_theta+k_h*N_theta);
	  connectivity[k+1] = (k_theta+1+k_h*N_theta);
	  connectivity[k+2] = (k_theta+(k_h+1)*N_theta);

	  //other way
	  connectivity[k+3] = ((k_theta+1)+k_h*N_theta);
	  connectivity[k+4] = ((k_theta+1)+(k_h+1)*N_theta);
	  connectivity[k+5] = (k_theta+(k_h+1)*N_theta);

	  //	  printf("%d,%d\n",k+5,3*N_triangle);

	  k+=6;

	}
    }



  return ok;



}

int Mesh_tool::create_torus(int N_theta,int N_phi,float R_ext,float R_int)
{
  int ok=0;
  int k=0;
  int k_theta=0,k_phi=0;
  float theta=0.0,phi=0.0;

  N_vertex = N_theta*N_phi;
  N_triangle = 2*(N_theta-1)*(N_phi-1);

  ok += allocate_mesh();
  if(ok!=0)
    exit(-1);
  
  for(k_theta=0;k_theta<N_theta;k_theta++)
    {
      //current theta
      theta = (float)k_theta*2*PI/(N_theta-1);
      for(k_phi=0;k_phi<N_phi;k_phi++)
	{
	  //current phi
	  phi = (float)k_phi*2*PI/(N_phi-1);
	  //position of the vertex vector
	  k = 3*(k_theta+k_phi*N_theta);

	  vertex[k]   = (R_ext+R_int*cos(phi))*cos(theta);
	  vertex[k+1] = (R_ext+R_int*cos(phi))*sin(theta);
	  vertex[k+2] = R_int*sin(phi);

	}
    }


  for(k=0,k_theta=0;k_theta<N_theta-1;k_theta++)
    {
      for(k_phi=0;k_phi<N_phi-1;k_phi++)
	{

	  //first way
	  connectivity[k] = (k_theta+k_phi*N_theta);
	  connectivity[k+1] = (k_theta+1+k_phi*N_theta);
	  connectivity[k+2] = (k_theta+(k_phi+1)*N_theta);

	  //other way
	  connectivity[k+3] = ((k_theta+1)+k_phi*N_theta);
	  connectivity[k+4] = ((k_theta+1)+(k_phi+1)*N_theta);
	  connectivity[k+5] = (k_theta+(k_phi+1)*N_theta);

	  k+=6;

	}
    }



  return ok;
}



int Mesh_tool::create_cone(int N_1,int N_2,float r_min,float r_max,float h)
{
  int ok=0;
  int k=0;
  int k_1=0,k_2=0;
  float u1=0.0,u2=0.0;

  N_vertex = N_1*N_2;
  N_triangle = 2*(N_1-1)*(N_2-1);

  ok += allocate_mesh();
  if(ok!=0)
    exit(-1);
  
  for(k_1=0;k_1<N_1;k_1++)
    {
      //current theta
      u1 = (float)k_1*2*PI/(N_1-1);
      for(k_2=0;k_2<N_2;k_2++)
	{
	  //current phi
	  u2 = (float)k_2/(N_2-1);
	  //position of the vertex vector
	  k = 3*(k_1+k_2*N_1);

	  

	  vertex[k]   = (r_min+(r_max-r_min)*u2)*cosf(u1);
	  vertex[k+1] = u2*h;
	  vertex[k+2] = (r_min+(r_max-r_min)*u2)*sinf(u1);

	}
    }


  for(k=0,k_1=0;k_1<N_1-1;k_1++)
    {
      for(k_2=0;k_2<N_2-1;k_2++)
	{

	  //first way
	  connectivity[k] = (k_1+k_2*N_1);
	  connectivity[k+1] = (k_1+1+k_2*N_1);
	  connectivity[k+2] = (k_1+(k_2+1)*N_1);

	  //other way
	  connectivity[k+3] = ((k_1+1)+k_2*N_1);
	  connectivity[k+4] = ((k_1+1)+(k_2+1)*N_1);
	  connectivity[k+5] = (k_1+(k_2+1)*N_1);

	  k+=6;

	}
    }



  return ok;
}









int Mesh_tool::set_parameterized_size(int u1,int u2)
{
  int ok=0;
  N_vertex = u1*u2;
  N_triangle = 2*(u1-1)*(u2-1);

  ok += allocate_mesh();

  return ok;
}


int Mesh_tool::build_parameterized_connectivity(int u1,int u2)
{
  int ok=0;

  if(connectivity==NULL)
    {
      printf("ERROR parameterized connectivity not build\n");
      exit(-1);
    }
  int k_1=0,k_2=0;
  int k_count=0;

  for(k_1=0;k_1<u1-1;k_1++)
    for(k_2=0;k_2<u2-1;k_2++)
      {
	k_count=2*3*(k_1+k_2*(u1-1));

	connectivity[k_count+0] = (k_1+0) + (k_2+0)*u1; 
	connectivity[k_count+1] = (k_1+1) + (k_2+0)*u1;
	connectivity[k_count+2] = (k_1+0) + (k_2+1)*u1;

	connectivity[k_count+3] = (k_1+1) + (k_2+0)*u1; 
	connectivity[k_count+4] = (k_1+1) + (k_2+1)*u1;
	connectivity[k_count+5] = (k_1+0) + (k_2+1)*u1;
      }



  return ok;
}

//fill one vertex
int Mesh_tool::fill_vertex(int k1,int k2,int N_1,float *values)
{
  int ok=0;

  if(vertex==NULL)
    exit(-1);
  if(k1+k2*N_1>N_vertex)
    {
      cout<<"ERROR cannot fill this vertex: too large\n";
      exit(-1);
    }



  //fill the value of the vertex
  int k_dim=0;
  for(k_dim=0;k_dim<3;k_dim++)
    vertex[3*(k1+k2*N_1)+k_dim] = values[k_dim];

  return ok;

}


int Mesh_tool::enable_parameterized_texture(int u1,int u2)
{
  int ok=0;

  if(texture_coordinates!=NULL)
    {
      delete[] texture_coordinates;
      texture_coordinates=NULL;
    }

  //allocation of the texture coordinate
  if(u1>0 && u2>0)
    texture_coordinates = new float[2*u1*u2];

  if(texture_coordinates==NULL)
    {
      cout<<"ERROR in allocation texture coordinate\n";
      exit(-1);
    }

  return ok;
}


int Mesh_tool::map_linear_texture_coordinate(int u1,int u2,int scale1,int scale2)
{
  int ok=0;

  if(texture_coordinates!=NULL)
    {
      delete []texture_coordinates;
      texture_coordinates = NULL;
    }

  if(texture_coordinates==NULL)
    ok += enable_parameterized_texture(u1,u2);

  int k_1=0,k_2=0;
  for(k_1=0;k_1<u1;k_1++)
    {
      for(k_2=0;k_2<u2;k_2++)
	{
	  texture_coordinates[2*(k_1+k_2*u1)+0] = (float)k_1/(u1-1)*scale1;
	  texture_coordinates[2*(k_1+k_2*u1)+1] = (float)k_2/(u2-1)*scale2;
	}
    }

  return ok;
}




int Mesh_tool::map_linear_multitexture_coordinate(int u1,int u2,int scale11,int scale12,int scale21,int scale22)
{
  int ok=0;

  if(texture_coordinates!=NULL)
    {
      delete []texture_coordinates;
      texture_coordinates = NULL;
    }

  if(texture_coordinates==NULL)
    ok += enable_parameterized_texture(2*u1,2*u2);

  int k_1=0,k_2=0;
  for(k_1=0;k_1<u1;k_1++)
    {
      for(k_2=0;k_2<u2;k_2++)
	{
	  texture_coordinates[2*(k_1+k_2*u1)+0] = (float)k_1/(u1-1)*scale11;
	  texture_coordinates[2*(k_1+k_2*u1)+1] = (float)k_2/(u2-1)*scale12;

	  texture_coordinates[2*(k_1+k_2*u1)+0+2*u1*u2] = (float)k_1/(u1-1)*scale21;
	  texture_coordinates[2*(k_1+k_2*u1)+1+2*u1*u2] = (float)k_2/(u2-1)*scale22;

	}
    }

  return ok;
}


int Mesh_tool::load_vertex(int _N_vertex,float *_vertex,int _N_triangle,int *_connectivity)
{
  int ok=0;

  N_vertex = _N_vertex;
  N_triangle = _N_triangle;

  ok += allocate_mesh();  

  int k=0;
  int k_dim=0;
  for(k=0;k<N_vertex;k++)
    for(k_dim=0;k_dim<3;k_dim++)
      vertex[3*k+k_dim] = _vertex[3*k+k_dim];

  for(k=0;k<N_triangle;k++)
    for(k_dim=0;k_dim<3;k_dim++)
      connectivity[3*k+k_dim] = _connectivity[3*k+k_dim];

  return ok;
}

int Mesh_tool::load_vertex(int _N_vertex,float *_vertex,float *_texture_coordinates,int _N_triangle,int *_connectivity)
{
  int ok=0;

  N_vertex = _N_vertex;
  N_triangle = _N_triangle;

  ok += allocate_mesh();
  texture_coordinates = new float[2*N_vertex];
  if(texture_coordinates==NULL)
    {
      cout<<"ERROR allocation of texture coordinate in load vertex\n";
      exit(-1);
    }

  int k=0;
  int k_dim=0;
  for(k=0;k<N_vertex;k++)
    {
      for(k_dim=0;k_dim<3;k_dim++)
	vertex[3*k+k_dim] = _vertex[3*k+k_dim];
      for(k_dim=0;k_dim<2;k_dim++)
	texture_coordinates[2*k+k_dim] = _texture_coordinates[2*k+k_dim];
    }
  for(k=0;k<N_triangle;k++)
    for(k_dim=0;k_dim<3;k_dim++)
      connectivity[3*k+k_dim] = _connectivity[3*k+k_dim];
  return ok;
}

int Mesh_tool::build_spherical_texture_coordinate()
{
  int ok=0;

  //delete in case
  if(texture_coordinates!=NULL)
    {
      delete[] texture_coordinates;
      texture_coordinates=NULL;
    }

  //new
  texture_coordinates = new float[2*N_vertex];
  if(texture_coordinates==NULL)
    {
      cout<<"ERROR allocation texture coordinate in spherical texture mapping\n";
      exit(-1);
    }

  int k_vertex=0;
  float x=0.0,y=0.0,z=0.0;
  float theta=0.0,phi=0.0;
  for(k_vertex=0;k_vertex<N_vertex;k_vertex++)
    {
      x = vertex[3*k_vertex+0];
      y = vertex[3*k_vertex+1];
      z = vertex[3*k_vertex+2];

//       theta = acosf(z/powf(x*x+y*y+z*z,0.5))/(PI);
//       phi = atan(y/x)/(PI);
//       theta = powf(x*x+z*z,0.5);
//       phi = y;

      theta=x;
      phi=y;

      texture_coordinates[2*k_vertex+0] = theta;
      texture_coordinates[2*k_vertex+1] = phi;
    }
  return ok;
}





int Mesh_tool::build_cylindrical_texture_coordinate(float L1,float L2)
{
  int ok=0;

  //delete in case
  if(texture_coordinates!=NULL)
    {
      delete[] texture_coordinates;
      texture_coordinates=NULL;
    }

  //new
  texture_coordinates = new float[2*N_vertex];
  if(texture_coordinates==NULL)
    {
      cout<<"ERROR allocation texture coordinate in spherical texture mapping\n";
      exit(-1);
    }

  int k_vertex=0;
  float x=0.0,y=0.0,z=0.0;
  float theta=0.0,phi=0.0;
  for(k_vertex=0;k_vertex<N_vertex;k_vertex++)
    {
      x = vertex[3*k_vertex+0];
      y = vertex[3*k_vertex+1];
      z = vertex[3*k_vertex+2];

      theta = fabs(atan2(z,x))/(2*PI);
      phi = (y+1)/2;

      texture_coordinates[2*k_vertex+0] = theta*L1;
      texture_coordinates[2*k_vertex+1] = phi*L2;
    }
  return ok;
}


int Mesh_tool::build_xz_texture_coordinate(float L1,float L2)
{
  int ok=0;

  //delete in case
  if(texture_coordinates!=NULL)
    {
      delete[] texture_coordinates;
      texture_coordinates=NULL;
    }

  //new
  texture_coordinates = new float[2*N_vertex];
  if(texture_coordinates==NULL)
    {
      cout<<"ERROR allocation texture coordinate in spherical texture mapping\n";
      exit(-1);
    }

  int k_vertex=0;
  float x=0.0,y=0.0,z=0.0;
  float theta=0.0,phi=0.0;
  for(k_vertex=0;k_vertex<N_vertex;k_vertex++)
    {
      x = vertex[3*k_vertex+0];
      y = vertex[3*k_vertex+1];
      z = vertex[3*k_vertex+2];

      theta = x;
      phi = z;

      texture_coordinates[2*k_vertex+0] = theta*L1;
      texture_coordinates[2*k_vertex+1] = phi*L2;
    }
  return ok;
}
