
#ifndef _Mesh_tool_h
#define _Mesh_tool_h

#include <iostream>
#include <string>
#include <cmath>
#include "library.h"

using namespace std;



#define MAX_SIZE_EXTENSION 15
#define SIZE_BUFFER 100
#define COLOR_XYZ 1
#define PI 3.14159






class Mesh_tool
{

 public:

  Mesh_tool();
  ~Mesh_tool();

  int load_mesh_file(const char* filename);
  int load_vertex(int N_vertex,float *vertex,int N_triangle,int *connectivity);
  int load_vertex(int N_vertex,float *vertex,float *texture_coordinates,int N_triangle,int *connectivity);

  //parameterized Mesh
  int set_parameterized_size(int u1,int u2);
  int enable_parameterized_texture(int u1,int u2);
  int map_linear_texture_coordinate(int u1,int u2,int scale1,int scale2);
  int map_linear_multitexture_coordinate(int u1,int u2,int scale11,int scale12,int scale21,int scale22);
  int build_parameterized_connectivity(int u1,int u2);
  int fill_vertex(int k1,int k2,int N_1,float *values);
  int build_spherical_texture_coordinate();
  int build_cylindrical_texture_coordinate(float L1,float L2);
  int build_xz_texture_coordinate(float L1,float L2);

  int build_normal();
  int fill_color(float *_color);
  int fill_color(float r,float g,float b);
  int fill_color(int type);
  int normalize_size();


  int scale(float *scale);
  int scale(float scale);
  int translate(float *translate);
  int translate(float t_x,float t_y,float t_z);
  int rotate(float angle,float *axe);
  int rotate(float angle,float r_x,float r_y,float r_z);


  //output
  float *get_vertex();
  float *get_normal();
  float *get_color();
  float *get_texture_coordinate();
  int *get_connectivity();
  int get_N_vertex();
  int get_N_triangle();


  int create_torus(int N_theta,int N_phi,float R_ext,float R_int);
  int create_cylinder(int N_theta,int N_h,float radius,float H);
  int create_cone(int N_1,int N_2,float r_min,float r_max,float h);

 private:

  int destroy();
  int load_ply_file(const char* filename);

  int read_ply_header(FILE *fid,const char* filename);
  int read_ply_data(FILE *fid,const char*filename);

  int allocate_mesh();

  int fill_rotation_matrix(float angle,float *axe,float *rotation_matrix);
  int matrix_product(float *x,float *M,float *y);

  int N_triangle;
  int N_vertex;

  float *vertex;
  float *normal;
  float *color;
  int *connectivity;

  float *texture_coordinates;
};

#endif
