#include "Rabit.h"

Rabit::Rabit()
{
  X[0] = 7.09;
  X[1] = 1.9;
  X[2] = 7.403*4/3;
}

Rabit::~Rabit()
{
  if(glIsList(draw_list))
    glDeleteLists(draw_list,1);

}

int Rabit::load_mesh(const char* filename)
{
  int ok=0;

  mesh.load_mesh_file(filename);
  mesh.fill_color(0.3,0.3,0.3);
  mesh.normalize_size();
  mesh.build_cylindrical_texture_coordinate(5,5);
  mesh.scale(0.03);
  mesh.build_normal();
  mesh.translate(X[0],X[1],X[2]);



  return ok;
}


int Rabit::draw(Position *position)
{
  float R2=2*2;

  //if close enough, draw it
  if( (position->get_x()-X[0])*(position->get_x()-X[0])+(position->get_z()-X[2])*(position->get_z()-X[2])<R2 )
    glCallList(draw_list);

  return 0;
}


int Rabit::set_texture(const char* texture_path)
{
  int ok=0;

  //**********************//
  //texture 
  //**********************//

  glEnable (GL_TEXTURE_2D);
  
  Texture tex;
  ok += tex.load_picture(texture_path);


  glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
  glGenTextures (1, &texture_number);
  glBindTexture (GL_TEXTURE_2D, texture_number);

  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

  gluBuild2DMipmaps (GL_TEXTURE_2D, 4, tex.get_Nx(), tex.get_Ny(), GL_RGBA,
                         GL_UNSIGNED_BYTE, tex.get_data());
  glTexEnvi (GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_REPLACE);



  ok += build_draw_list();
  
  return ok;
}

int Rabit::build_draw_list()
{
  int ok=0;


  if(glIsList(draw_list))
    glDeleteLists(draw_list,1);
  draw_list = glGenLists(1);
  glNewList(draw_list,GL_COMPILE);

  glColor4f(0.6,0.6,0.7,1.0);
  glEnable(GL_TEXTURE_2D);
  glBindTexture(GL_TEXTURE_2D,texture_number);
  
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);
  
  glVertexPointer(3,GL_FLOAT,0,mesh.get_vertex());
  glNormalPointer(GL_FLOAT,0,mesh.get_normal());
  glTexCoordPointer(2,GL_FLOAT,0,mesh.get_texture_coordinate());
  
  glDrawElements(GL_TRIANGLES,mesh.get_N_triangle()*3,GL_UNSIGNED_INT,mesh.get_connectivity());
  
  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);
  
  glColor4f(1.0,1.0,1.0,1.0);
  glDisable(GL_TEXTURE_2D);

  glEndList();


  return ok;
}
