#include "Water_surface.h"


Water_surface::Water_surface()
{
  vertex = NULL;
  normal = NULL;
  texture_coord = NULL;
  connectivity = NULL;

  N_x=0;
  N_y=0;
  texture_number=0;
}

Water_surface::~Water_surface()
{
  destroy();
}

int Water_surface::destroy()
{
  N_x=0;
  N_y=0;
  if(vertex!=NULL)
    {
      delete[] vertex;
      vertex=NULL;
    }
  if(normal!=NULL)
    {
      delete[] normal;
      normal=NULL;
    }
  if(texture_coord!=NULL)
    {
      delete[] texture_coord;
      texture_coord=NULL;
    }
  if(connectivity!=NULL)
    {
      delete[] connectivity;
      connectivity=NULL;
    }

  return 0;
}


int Water_surface::set_size(int _N_x,int _N_y,float scale_1,float scale_2)
{
  int ok=0;

  ok+=destroy();

  N_x = _N_x;
  N_y = _N_y;

  if(N_x>0 && N_y>0)
    {
      vertex = new float[N_x*N_y*3];
      normal = new float[N_x*N_y*3];
      texture_coord = new float[N_x*N_y*2];
      connectivity = new int[2*3*(N_x-1)*(N_y-1)];

      if(vertex==NULL || normal==NULL || texture_coord==NULL || connectivity==NULL)
	{
	  cout<<"ERROR allocation failed on water surface\n";
	  return (-1);
	}
      
      //fill the pointers
      int k_x=0,k_y=0;
      for(k_x=0;k_x<N_x;k_x++)
	{
	  for(k_y=0;k_y<N_y;k_y++)
	    {
	      vertex[3*(k_x+k_y*N_x)+0] = (float)k_x/(N_x-1)*scale_1;
	      vertex[3*(k_x+k_y*N_x)+1] = WATER_LEVEL;
	      vertex[3*(k_x+k_y*N_x)+2] = (float)k_y/(N_y-1)*scale_2;

	      normal[3*(k_x+k_y*N_x)+0] = 0;
	      normal[3*(k_x+k_y*N_x)+1] = 1;
	      normal[3*(k_x+k_y*N_x)+2] = 0;

	      texture_coord[2*(k_x+k_y*N_x)+0] = (float)k_x/(N_x-1)*LAMBDA_WATER;
	      texture_coord[2*(k_x+k_y*N_x)+1] = (float)k_y/(N_y-1)*LAMBDA_WATER;
	    }
	}
      //fill connectivity
      int count=0;
      for(k_x=0;k_x<N_x-1;k_x++)
	{
	  for(k_y=0;k_y<N_y-1;k_y++)
	    {

	      connectivity[count+0] = (k_x+0) + (k_y+0)*N_x;
	      connectivity[count+1] = (k_x+1) + (k_y+0)*N_x;
	      connectivity[count+2] = (k_x+0) + (k_y+1)*N_x;

	      connectivity[count+3] = (k_x+1) + (k_y+0)*N_x;
	      connectivity[count+4] = (k_x+1) + (k_y+1)*N_x;
	      connectivity[count+5] = (k_x+0) + (k_y+1)*N_x;

	      count+=6;
	    }
	}

      return ok;

    }
  else
    return (-1);
}



int Water_surface::set_texture(const char* texture_path)
{
  int ok=0;

  //**********************//
  //texture 
  //**********************//

  glEnable (GL_TEXTURE_2D);
  
  Texture tex;
  ok += tex.load_picture(texture_path);


  glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
  glGenTextures (1, &texture_number);
  glBindTexture (GL_TEXTURE_2D, texture_number);

  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR_MIPMAP_LINEAR);
  glTexParameteri (GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR);

  gluBuild2DMipmaps (GL_TEXTURE_2D, 4, tex.get_Nx(), tex.get_Ny(), GL_RGBA,
                         GL_UNSIGNED_BYTE, tex.get_data());
  glTexEnvi (GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_REPLACE);
  
  return ok;
}



int Water_surface::draw()
{

  //set color
  glColor4f(0.2,0.2,0.4,0.7);


  //enable texture
  glEnable(GL_BLEND);
  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

  glEnable (GL_TEXTURE_2D);
  glBindTexture (GL_TEXTURE_2D, texture_number);


  //enable memory placement
  glEnableClientState(GL_VERTEX_ARRAY);
  glEnableClientState(GL_NORMAL_ARRAY);
  glEnableClientState(GL_TEXTURE_COORD_ARRAY);

  glVertexPointer(3,GL_FLOAT,0,vertex);
  glTexCoordPointer(2,GL_FLOAT,0,texture_coord);
  glNormalPointer(GL_FLOAT,0,normal);



  //*****************************//
  //DRAW
  //*****************************//
  
  //draw many patch
  int k_x=0,k_y=0;
  for(k_x=-1;k_x<=3;k_x++)
    for(k_y=-1;k_y<=3;k_y++)
      {
	glPushMatrix();
	glTranslatef(10*k_x,0,10*k_y);
	glDrawElements(GL_TRIANGLES,3*2*(N_x-1)*(N_y-1),GL_UNSIGNED_INT,connectivity);
	glPopMatrix();
      }




  //*****************************//
  //\DRAW
  //*****************************//


  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);
  glDisableClientState(GL_TEXTURE_COORD_ARRAY);



  glDisable(GL_TEXTURE_2D);
  glDisable(GL_BLEND);



  return 0;
}


int Water_surface::animate(int kt)
{
  int ok=0;

  float t=(float)kt/300;

  int k_x=0,k_y=0;
  float norm=0;
  int k_dim=0;
  float px=0.0,py=0.0;
  for(k_x=0;k_x<N_x;k_x++)
    {
      px=(float)k_x/(N_x-1);
      for(k_y=0;k_y<N_y;k_y++)
	{
	  py=(float)k_y/(N_y-1);


	  //change normal
	  normal[3*(k_x+k_y*N_x)+0]=0.25*powf(sin(2*t-py*2*PI),2);
	  normal[3*(k_x+k_y*N_x)+2]=0.25*2*powf(sin(3*t-px*2*PI),2);

	  //renormalization
	  for(norm=0,k_dim=0;k_dim<3;k_dim++)
	    norm += normal[3*(k_x+k_y*N_x)+k_dim]*normal[3*(k_x+k_y*N_x)+k_dim];
	  norm = powf(norm,(float)0.5);
	  for(k_dim=0;k_dim<3;normal[3*(k_x+k_y*N_x)+k_dim]/=norm,k_dim++);


	  //change texture
	  texture_coord[2*(k_x+k_y*N_x)+0] = LAMBDA_WATER*(px+0.1/10*cos(10*t-px*(2*PI)*3)+0.1*cos(2*t-py*PI*2));
	  texture_coord[2*(k_x+k_y*N_x)+1] = LAMBDA_WATER*(py+0.1/10*cos(10*t-py*(2*PI)*3)+0.05*cos(8*t-py*2*PI));

	}
    }


  return ok;
}


