#include "library.h"
#include "OpenGL_interface.h"
#include "Ground.h"
#include "Position.h"
#include "Water_surface.h"
#include "Transparent_elements.h"
#include "Sky_box.h"
#include "Castle.h"
#include "Rabit.h"
#include "Dragon.h"
#include "Sword.h"
#include "Map.h"
#include "Grass.h"

//functions
static void display_callback();
static void reshape_callback (int width, int height);
static void keyboard_callback(unsigned char key,int x,int y);
static void idle_callback();
static void passive_mouse_callback(int x, int y); 




//global variables
OpenGL_interface opengl_interface;
Ground ground;
Position position;
Water_surface water;
Transparent_elements transparent_elements;
Sky_box sky_box;
Castle castle;
Rabit rabit;
Sword sword;
Dragon dragon;
Map top_map;
Grass grass;


int kt=0;
int k_step=0;
int special_key=0;




int main(int argc,char **argv)
{
  printf("**hello**\n\n");



  //init
  printf("INIT OpenGL ...\n");
  opengl_interface.init_interface(argc,argv);


  //LOAD GROUND
  //****************************//
  printf("INIT Ground ...\n");
  ground.set_height_field_texture(TEXTURE_GROUND_COLOR_PATH,TEXTURE_GROUND_GRASS_PATH);
  ground.set_scale(40,8,40);
  ground.build(50,50,TEXTURE_GROUND_PATH);



  //LOAD TREES
  //****************************//
  printf("INIT Trees ...\n");
  transparent_elements.set_texture(TEXTURE_BILLBOARD_PATH_1,TEXTURE_BILLBOARD_PATH_2,TEXTURE_BILLBOARD_PATH_3,TEXTURE_TRANSPARENT_GRASS_PATH);

  //uncomment to load random trees
  //  transparent_elements.set_number(700,200,200,2000);
  //  transparent_elements.set_position(&ground);

  //load trees positions from file
  transparent_elements.load_transparent_file();




  //LOAD WATER
  //****************************//
  printf("INIT Water ...\n");
  water.set_size(40,40,10,10);
  water.set_texture(TEXTURE_WATER_PATH);

  //LOAD CASTLE
  //****************************//
  printf("INIT Castle ...\n");
  castle.set_texture();
  castle.build_castle();

  //LOAD RABIT
  //****************************//
  printf("INIT Rabit ...\n");
  rabit.load_mesh(MESH_RABIT_PATH);
  rabit.set_texture(TEXTURE_MESH3D_PATH);

  //LOAD DRAGON
  //****************************//
  printf("INIT Dragon ...\n");
  dragon.load_mesh(MESH_DRAGON_PATH);
  dragon.set_texture(TEXTURE_MESH3D_PATH);

  //LOAD SKY BOX
  //****************************//
  printf("INIT Sky Box ...\n");
  sky_box.set_texture();

  //LOAD SWORD
  //****************************//
  printf("INIT Sword ...\n");
  sword.set_texture();

  //LOAD GRASS
  //****************************//
  printf("INIT Grass ...\n");
  grass.set_texture(TEXTURE_GRASS_PATH);
  grass.set_number(200000);
  grass.set_position(&ground);

  //LOAD MAP
  //****************************//
  printf("INIT Map ...\n");
  top_map.set_texture(TEXTURE_GROUND_PATH);



  printf("\nEND INIT OK\n\n");


  printf("\n* control with touch: 'z','s','d','q' to move\n* 't' and 'g' for up and down\n* the angle is set with the mouse\n* SPACE for action\n* 'Q' to exit\n\n");

  //callback
  glutDisplayFunc (display_callback);
  glutReshapeFunc (reshape_callback);
  glutKeyboardFunc(keyboard_callback);
  glutIdleFunc(idle_callback);
  glutPassiveMotionFunc(passive_mouse_callback); 

  //run
  printf("**RUN**\n\n");
  glutMainLoop ();


  return 0;
}


static void display_callback()
{



  //set reflexion matrix
  float reflect[16] =
    {
      1, 0, 0, 0,
      0,-1, 0, 0,
      0, 0, 1, 0,
      0, 0, 0, 1
    };
  //set cutting plane
  GLdouble clip[4];

  //view matrix
  GLfloat matrix[16];





  //set viewport
  glViewport (0, 0, position.get_width(), position.get_height());
  //clear
  glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);



  //REFLECTION WORLD
  //***************************//

  //1.Projection Camera
  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective (45, 1, 0.01,100);


  //2.World
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity ();
  glRotatef(position.get_alpha(),0.0,1.0,0.0);
  glRotatef(position.get_beta(),cos(position.get_alpha()/180*PI),0.0,sin(position.get_alpha()/180*PI));
  glTranslatef(-position.get_x(),-position.get_y(),-position.get_z());

  //cut plane
  clip[0] = 0.0;
  clip[1] = -1;
  clip[2] = 0;
  clip[3] = WATER_LEVEL;
  glEnable (GL_CLIP_PLANE0);
  glClipPlane (GL_CLIP_PLANE0, clip);
  glColor4f(0.8,0.8,1.0,1.0);

  //load matrix and reflect
  glGetFloatv(GL_MODELVIEW_MATRIX,matrix);
  glTranslatef(0.0,WATER_LEVEL*2,0.0);
  transparent_elements.set_matrix(matrix,&position);
  glMultMatrixf (reflect);

  //sky_box
  glPushMatrix();
  glTranslatef(position.get_x(),WATER_LEVEL,position.get_z());
  sky_box.draw();
  glPopMatrix();

  //draw
  glColor4f(0.13,0.13,0.20,1.0);
  ground.draw();
  dragon.draw(&position);
  //  grass.draw(&position);//do not draw the grass reflexion
  transparent_elements.draw(kt);
  glDisable(GL_CLIP_PLANE0);






  //WATER
  //***********************************//
  water.animate(kt);
  water.draw();






  //NORMAL WORLD
  //***********************************//


  //The projection matrix is already set

  //modelview
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity ();
  glRotatef(position.get_alpha(),0.0,1.0,0.0);
  glRotatef(position.get_beta(),cos(position.get_alpha()/180*PI),0.0,sin(position.get_alpha()/180*PI));
  glTranslatef(-position.get_x(),-position.get_y(),-position.get_z());


  //cut plane
  //plane y-WATER_LEVEL=0
  clip[0] = 0.0;
  clip[1] = 1;
  clip[2] = 0;
  clip[3] = -WATER_LEVEL;
  glEnable (GL_CLIP_PLANE0);
  glClipPlane (GL_CLIP_PLANE0, clip);
  glColor4f(1.0,1.0,1.0,1.0);

  //get matrix
  glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

  //draw

  //sky_box
  glPushMatrix();
  glTranslatef(position.get_x(),WATER_LEVEL,position.get_z());
  sky_box.draw();
  glPopMatrix();

  //ground
  ground.draw();
  //caslte
  castle.draw(kt);
  //rabit
  rabit.draw(&position);
  //dragon
  dragon.draw(&position);
  //grass
  grass.draw(matrix,&position,kt);
  //transparent trees and animals
  transparent_elements.set_matrix(matrix,&position);
  transparent_elements.draw(kt);

  glDisable(GL_CLIP_PLANE0);


  //sword
  sword.draw(k_step,&position);
  //map
  top_map.draw(&position);




  //redraw
  glFlush ();
  glutSwapBuffers ();


}





static void reshape_callback (int width, int height)
{
  position.set_size(width,height);
  glViewport (0, 0, width, height);
  glutPostRedisplay ();
}



static void keyboard_callback(unsigned char key,int x,int y)
{

  //speed of displacement
  float dL=0.013;

  float current_x=0.0,current_y=0.0,current_z=0.0;
  float alpha=0.0,beta=0.0;

  float h;

  //useless but avoid the warning message
  x=0;
  y=0;

  current_x = position.get_x();
  current_y = position.get_y();
  current_z = position.get_z();

  alpha = position.get_alpha();
  beta  = position.get_beta();

  special_key = glutGetModifiers();


  //  printf("%f,%f,%f\n",current_x,current_y,current_z);
  switch(key)
    {
    case 'Q'://quit
      printf("\nExit OK\n\n");
      exit(0);
      break;
    case 'z'://forward
      current_x+=dL*sin(alpha*PI/180);
      current_z-=dL*cos(alpha*PI/180)+0.0005*cos((float)k_step/4);
      h=ground.get_height(current_x,current_z)+0.006*cos((float)k_step/3);
      position.set_position(current_x,h+0.1,current_z);
      k_step++;
      break;
    case 's'://backward
      current_x-=dL*sin(alpha*PI/180);
      current_z+=dL*cos(alpha*PI/180);
      h=ground.get_height(current_x,current_z);
      position.set_position(current_x,h+0.1,current_z);
      break;
    case 'd'://right
      current_x+=dL*cos(alpha*PI/180);
      current_z+=dL*sin(alpha*PI/180);
      h=ground.get_height(current_x,current_z);
      position.set_position(current_x,h+0.1,current_z);
      break;
    case 'q'://left
      current_x-=dL*cos(alpha*PI/180);
      current_z-=dL*sin(alpha*PI/180);
      h=ground.get_height(current_x,current_z);
      position.set_position(current_x,h+0.1,current_z);
      break;
    case 't'://up
      position.set_position(current_x,current_y+0.1,current_z);
      break;
    case 'g'://down
      position.set_position(current_x,current_y-0.1,current_z);
      break;

    case GLUT_SPACE://space
      position.enable_fight();
      break;
    }


  glutPostRedisplay();

}

static void idle_callback()
{
  //increment current time
  kt++;
  glutPostRedisplay();
}


static void passive_mouse_callback(int x, int y)
{
  //could do a better job than absolute coordinates...
  position.set_angle((float)x*0.25*PI,(float)y*0.25*PI);  
}
